import requests
import json


class GraphDB(object):

    def __init__(self, base_url, credentials=None):
        """

        """
        self.base_url = base_url
        self.credentials = credentials

    def is_running(self):
        """
        Check if GraphDB server is running
        """
        if requests.get(self.base_url + "rest/repositories", auth=self.credentials):
            # returns True for all Status Codes 200-400
            return True
        else:
            print("Graph_DB is not running or access is not granted")
            return False

    def clear_repository(self, repository):
        """
        Clear contents of currently selected repository
        """
        url = self.base_url + 'repositories/' + repository + '/' + 'statements'
        response = requests.request(method='DELETE', url=url, auth=self.credentials)
        return response

    def upload_rdf_file(self, repository, graph, rdf_file, rdf_as_file=False):
        """
        Upload rdf from turtle file or variable into selected graph from repository
        """
        url = self.base_url + 'repositories/' + repository + '/' + 'statements'
        headers = {
            'Content-type': 'application/x-turtle'
        }
        params = {'graph': graph}
        if rdf_as_file:
            data = open(rdf_file, 'r', encoding='utf-8').read()
        else:
            data = rdf_file
        response = requests.request(method='PUT', url=url, headers=headers, params=params,
                                    data=data, auth=self.credentials)
        return response

    def download_graph(self, repository, graph, download_format='application/rdf+json',
                       save_as_file=False, export_filename='data.json'):

        url = self.base_url + 'rest/explore/graph'
        params = {
            'uri': graph,
            'role': 'context',
            'inference': 'all',
            'bnodes': True,
            'sameAs': True
        }
        # params = json.dumps(params, separators=('&', '='))
        headers = {
            'Accept': download_format,
            'X-GraphDB-Repository': repository,
        }
        response = requests.request(method='GET', url=url, params=params, headers=headers,
                                    auth=self.credentials)
        download = json.loads(response.text)
        if save_as_file:
            with open(export_filename, 'w') as f:
                json.dump(download, f)
        return download



class OPG:
    def __init__(self):
        self.Name = ""
        self.ListOfNodes = []
        self.ListPGEdges = []

    def add_node(self):
        node = Node()
        self.ListOfNodes.append(node)
        return node


class Node:
    def __init__(self):
        self.NodeNr = 0
        self.NodeID = ""
        self.ListCPDCCombinations = []

    def add_cpdc_combination(self):
        combination = CPDCCombination()
        self.ListCPDCCombinations.append(combination)
        return combination


class CPDCCombination:
    def __init__(self):
        self.ListActorialResources = []
        self.Name = ""
        self.CPDCID = ""

    def add_actorial_resource(self, resource):
        self.ListActorialResources.append(resource)



import requests
from pathlib import Path
import json
from . import utils


class OntoRefine:
    """
    Class to access the REST API of the Onto Refine Project of a GraphDB Instance.
    The API is not officially released as REST API, so changes might occur without notice.
    """

    def __init__(self, base_url, credentials, onto_refine_path='orefine/command/core/'):
        self.base_url = base_url
        self.onto_refine_path = onto_refine_path
        self.onto_refine_base_url = base_url + onto_refine_path
        self.credentials = credentials

    def get_new_csrf_token(self):
        """ Token to be added to most API Requests """

        url = self.onto_refine_base_url + "get-csrf-token"
        payload = ""
        headers = {}

        response = requests.request("GET", url, headers=headers, data=payload, auth=self.credentials)
        j = json.loads(response.text)
        return j["token"]

    def get_current_project_ids(self):
        """ Return list of all project IDs."""
        return list(self.get_all_projects_meta_data()["projects"].keys())

    def get_all_projects_meta_data(self, export_as_json=False):
        """ Get all projects Meta-Data (and ids) """

        url = self.onto_refine_base_url + "get-all-project-metadata?csrf_token=" + self.get_new_csrf_token()

        headers = {
            'Accept': 'application/json',
            'cache-control': "no-cache"
        }

        response = requests.request("GET", url, headers=headers, auth=self.credentials)

        data = json.loads(response.text)
        if export_as_json:
            with open('tmp/all_open_refine_projects_meta_data.json', 'w') as f:
                json.dump(data, f, indent=4)

        return data

    def create_ontorefine_project(self, project_name, data_file, format_type="text/json", options=None):
        """format can be of (compare: https://docs.openrefine.org/technical-reference/openrefine-api)"""
        # compare https://github.com/paulmakepeace/refine-client-py/blob/master/google/refine/refine.py for options

        current_project_ids = set(self.get_current_project_ids())

        if options is None:
            options = utils.get_options_from_type(format_type)

        multipart_form_data = {
            'project-file': (project_name, open(data_file, 'rb')),
            'project-name': (None, project_name),
            'format': (None, format_type),
            'options': (None, json.dumps(options))
        }

        url = self.onto_refine_base_url + "create-project-from-upload"
        params = {
            'csrf_token': self.get_new_csrf_token()
        }
        requests.request(method="POST", url=url, params=params, files=multipart_form_data, auth=self.credentials)

        created_projects = list(set(self.get_current_project_ids()) - current_project_ids)
        if len(created_projects) > 0:
            return created_projects[0]
        else:
            print("Project could not be created!")
            return None

    def fill_down(self, project_id, column):
        """ Fill Down column with element"""
        url = self.onto_refine_base_url + 'fill-down'

        params = {
            'columnName': column,
            'project': project_id,
            'csrf_token': self.get_new_csrf_token()
        }

        data = {
            "engine": '{"facets": [], "mode": "record-based"}'
        }

        response = requests.request(method='POST', url=url, params=params, data=data, auth=self.credentials)
        print(json.loads(response.text)["historyEntry"]["description"])
        return response

    def apply_rdf_mapping(self, rdf_map_path, project_id, outputpath=None):
        """ Apply mapping to uploaded OntoRefine Project File and convert it to turtle file"""

        url = self.base_url + "rest/rdf-mapper/rdf/ontorefine:{}".format(project_id)
        
        rdf_map = json.loads(Path(rdf_map_path).read_text())

        headers = {
            'Accept': 'text/turtle;charset=UTF-8',
            'Content-Type': 'application/json'
        }

        response = requests.request("POST", url, headers=headers, data=json.dumps(rdf_map), auth=self.credentials)
        if outputpath:
            with open(outputpath, 'w') as f:
                f.write(response.text)

        return response.text



def get_options_from_type(format_type):
    """
    Get Options for OntoRefine upload based on format_type. format_type has to be supported by GraphDB
    (Currently only 'text/json' and 'text/line-based/*sv' implemented)
    """
    if format_type == 'text/json':
        options = {
            'recordPath': ["_"],
            'limit': -1,
            'trimStrings': False,
            'guessCellValueTypes': False,
            'storeEmptyStrings': False,
            'includeFileSources': False
        }
    elif format_type == 'text/line-based/*sv':
        options = {
            'separator': "\\t",
            'encoding': 'UTF-8',
            'ignoreLines': -1,
            'headerLines': 1,
            'skipDataLines': 0,
            'limit': -1,
            'storeBlankRows': False,
            'storeBlankCellsAsNulls': True,
            'trimStrings': True,
            'quoteCharacter': '"'
        }
    else:
        print("This format is not yet implemented or not supported")
        return None
    return options



class ResourceList:
    def add_resource(self):
        resource = Resource()
        self.ResourcesList.append(resource)
        return resource

    def get_resource_by_id(self, cpdid):
        for resource in self.ResourcesList:
            if resource.CPDID == cpdid:
                return resource
        return None


class Resource:
    def __init__(self):
        self.Name = ""
        self.CPDID = ""
        self.Reconfiguration = False
        self.oActorialResource = ActorialResource()
        # self.oSensorialResource = self.SensorialResource()


class ActorialResource:
    def __init__(self):
        self.ActuatorName = ""
        self.ActuatorID = ""
        self.ActuatorType = ""


class SensorialResource:
    def __init__(self):
        pass



import json
from .resource_list import ResourceList, Resource
from .opg_list import OPG
# from .osg_list import OSG


class ProductionSystem:
    def __init__(self):
        self.ResourcesList = []
        self.oPG = OPG()
        # self.oSG = OSG()

    def add_resource(self):
        resource = Resource()
        self.ResourcesList.append(resource)
        return resource



    def to_json(self):
        return json.dumps(self, default=lambda o: o.__dict__, sort_keys=True, indent=4)
